{
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::    Author: Philip Bairstow.                                                ::
::    E-Mail: Philip@pbairstow.com                                            ::
::    www: www.pbairstow.com                                                  ::
::    ICQ: 47850165                                                           ::
::                                                                            ::
::    04 June 2000.                                                           ::
::                                                                            ::
::    Version information:                                                    ::
::                                                                            ::
::    Version 1.0 - Gives user the ability to Read/Write String, Integer      ::
::                  and Boolean values entries TO/FROM the Registry.          ::
::                  Value is Stored in the Following Way:                     ::
::                                                                            ::
::                  RegistryBaseKey\CompanyName\ProgramName\ProgramVersion.   ::
::                                                                            ::
::    Version 1.2 - Added Read/Write Currency; Added Key Exists.              ::
::                  Added Read/Write String; Renamed old read/write           ::
::                  String to read/write EncryptedString;                     ::
::                  Added EasyRegistry Icon. Changed Encryption Procedure.     ::
::                                                                            ::
::                                                                            ::
::    If you use this product then you agree with the Following:              ::
::    THAT THE AUTHOR DOES NOT ACCEPT RESPONSABILITY FOR THE LOSS OR DAMAGE   ::
::    CAUSED EITHER DIRECTLY OR INDIRECTLY BY USING THIS SOFTWARE.            ::
::    YOU MAY NOT SELL OR PROFIT IN ANY WAY FROM THIS SOFTWARE WITHOUT THE    ::
::    WRITTEN PERMISSION FROM THE AUTHOR AND ANY ASSOCIATES.                  ::
::    THIS SOFTWARE MAY BE FREELY DISTRIBUTABLE AS ALONG AS ALL ORIGIONAL     ::
::    FILES ARE UNCHANGED AND PRESENT. YOU MUST REPORT ANY BUGS TO THE EMAIL  ::
::    ADDRESS IN THE README.TXT FILE.                                         ::
::                                                                            ::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
}

{$R Easyregistry.res}

unit EasyRegistry;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Registry;

type
  {Base Key Type Options: Note bkHKEY_DYN_DATA not implemented to as this key can not be accessed.}
  TRegBaseKey = (bkHKEY_CLASSES_ROOT, bkHKEY_CURRENT_USER, bkHKEY_LOCAL_MACHINE, bkHKEY_USERS, bkHKEY_CURRENT_CONFIG);
  TEasyRegistry = class(TComponent)
  private
    FReg: TRegistry;
  protected
    {Company Name}
    FCompanyName,
    {Program Name}
    FProgramName,
    {Program Version}
    FProgramVersion,
    {Path of Registry Entry}
    FBaseKey: String;
    {Selected Base Key}
    FRegBaseKey: TRegBaseKey;
    {Registry Base key}
    FRegistryBaseKey: HKEY;
    {Returns a HKEY Equivelent to the Value Of FRegBaseKeystry}
    Function SetBaseKey(Key: TRegBaseKey):HKEY;
    {Encryts String Values}
    Function Crypt(S: String; EncryptKey:Integer): string;
  public
    {Writes a Encrypted String Value to the Registry}
    Procedure WriteEncryptedString(Subkey,Value: String; EncryptionKey:Integer);
    {Returns a De-Crypted String Value From the Registry}
    Function ReadEncryptedString(Subkey: String; EncryptionKey:Integer): String;
    {Writes a String Value to the Registry}
    Procedure WriteString(Subkey,Value: String);
    {Returns a String Value From the Registry}
    Function ReadString(Subkey: String): String;
    {Writes a Integer to the Registry}
    Procedure WriteInteger(Subkey: String; Value: integer);
    {Reads a Integer From the Registry}
    Function ReadInteger(Subkey: String):Integer;
    {Writes a Boolean Value to the Registry}
    Procedure WriteBoolean(Subkey: String; Value: Boolean);
    {Reads a Boolean Value From the Registry}
    Function ReadBoolean(Subkey: String): Boolean;
    {Writes a Currency Value to the Registry}
    Procedure WriteCurrency(Subkey: String; Value: Currency);
    {Reads a Currency Value From the Registry}
    Function ReadCurrency(Subkey: String):Currency;
    {Determines if a key Exists}
    Function KeyExist(Subkey: String):Boolean;
  published
    {Sets the Registry Base Key}
    Property RegistryBaseKey: TRegBaseKey read FRegBaseKey write FRegBaseKey;
    {Sets the Value used in Encyption}
    Property CompanyName: String read FCompanyName write FCompanyName;
    {Sets the Program Version}
    Property ProgramVersion: String read FProgramVersion write FProgramVersion;
    {Sets the Program Name}
    Property ProgramName: String read FProgramName write FProgramName;
  end;

procedure Register;
var
  Temp: String;
    
implementation

procedure Register;
begin
  RegisterComponents('PBSoftware', [TEasyRegistry]);
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.SetBaseKey(Key: TRegBaseKey):HKEY;
begin
  If Key = bkHKEY_CLASSES_ROOT then Result := HKEY_CLASSES_ROOT else
  If Key = bkHKEY_CURRENT_USER then Result := HKEY_CURRENT_USER else
  If Key = bkHKEY_LOCAL_MACHINE then Result := HKEY_LOCAL_MACHINE else
  If Key = bkHKEY_USERS then Result := HKEY_USERS else
  If Key = bkHKEY_CURRENT_CONFIG then Result := HKEY_CURRENT_CONFIG else
  ERegistryException.Create('Invalid Key Type');
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.Crypt(S: String; EncryptKey:Integer): string;
var
  tmpstr: string;
  x: integer;
  b: byte;
begin
  tmpstr:= '';
  for x:= 1 to length(S) do
  begin
    b:= not (byte(S[x]) xor EncryptKey);
    tmpstr:= tmpstr + char(b);
  end;
  result:= tmpstr;
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.KeyExist(Subkey: String):Boolean;
begin
 FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    begin
      FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
      FReg := TRegistry.Create;
      Try
         FReg.RootKey := FRegistryBaseKey;
         FReg.CreateKey(FBaseKey);
         FReg.OpenKey(FBaseKey, False);
         Result := FReg.KeyExists(Subkey);
      Finally
         FReg.Free;
      end
  end;

  Except
     On E:ERegistryException do Raise;
   end;
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.ReadCurrency(Subkey: String):Currency;
begin
 FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    begin
      FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
      FReg := TRegistry.Create;
      Try
         FReg.RootKey := FRegistryBaseKey;
         FReg.CreateKey(FBaseKey);
         FReg.OpenKey(FBaseKey, False);
         Result := FReg.ReadCurrency(Subkey);
      Finally
         FReg.Free;
      end
  end;

  Except
     On E:ERegistryException do Raise;
    end;
end;

//------------------------------------------------------------------------------

Procedure TEasyRegistry.WriteCurrency(Subkey: String; Value: Currency);
begin
 FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    begin
      FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
      FReg := TRegistry.Create;
      Try
         FReg.RootKey := FRegistryBaseKey;
         FReg.CreateKey(FBaseKey);
         FReg.OpenKey(FBaseKey, False);
         FReg.WriteCurrency(Subkey, Value);
      Finally
         FReg.Free;
      end
  end;

  Except
     On E:ERegistryException do Raise;
   end;
end;

//------------------------------------------------------------------------------

Procedure TEasyRegistry.WriteString(Subkey, Value: String);
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    begin
      FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
      FReg := TRegistry.Create;
      Try
         FReg.RootKey := FRegistryBaseKey;
         FReg.CreateKey(FBaseKey);
         FReg.OpenKey(FBaseKey, False);
         FReg.WriteString(Subkey, Value);
      Finally
         FReg.Free;
      end
  end;

  Except
     On E:ERegistryException do Raise;
   end;
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.ReadString(Subkey: String): String;
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
    FReg := TRegistry.Create;
    Try
       FReg.Rootkey := FRegistryBaseKey;
       FReg.CreateKey(FBaseKey);
       FReg.OpenKey(FBaseKey, False);
       Result := FReg.ReadString(Subkey);
    Finally
       FReg.Free;
    end;

  Except
    On E:ERegistryException do Raise;
  end;
end;

//------------------------------------------------------------------------------

Procedure TEasyRegistry.WriteEncryptedString(Subkey, Value: String; EncryptionKey:Integer);
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    begin
      Temp := Crypt(Value, EncryptionKey);
      FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
      FReg := TRegistry.Create;
      Try
         FReg.RootKey := FRegistryBaseKey;
         FReg.CreateKey(FBaseKey);
         FReg.OpenKey(FBaseKey, False);
         FReg.WriteString(Subkey, Temp);
      Finally
         FReg.Free;
      end
  end;

  Except
    On E:ERegistryException do Raise;
   end;
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.ReadEncryptedString(Subkey: String; EncryptionKey:Integer): String;
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
    FReg := TRegistry.Create;
    Try
       FReg.Rootkey := FRegistryBaseKey;
       FReg.CreateKey(FBaseKey);
       FReg.OpenKey(FBaseKey, False);
       Temp := FReg.ReadString(Subkey);
       Result := Crypt(Temp, EncryptionKey);
    Finally
       FReg.Free;
    end;

  Except
    On E:ERegistryException do Raise;
  end;
end;

//------------------------------------------------------------------------------

Procedure TEasyRegistry.WriteInteger(Subkey: String; Value: Integer);
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
    FReg := TRegistry.Create;
      Try
        FReg.Rootkey := FRegistryBaseKey;
        FReg.CreateKey(FBasekey);
        FReg.OpenKey(FBaseKey, False);
        FReg.WriteInteger(SubKey, Value);
      Finally
        FReg.Free;
        end;
  Except
    On E:ERegistryException do Raise;
  end;
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.ReadInteger(SubKey: String): Integer;
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
     FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
     FReg := TRegistry.Create;
     Try
       FReg.Rootkey := FRegistryBaseKey;
       FReg.CreateKey(FBasekey);
       FReg.OpenKey(FBaseKey, False);
       Result := FReg.ReadInteger(Subkey);
     Finally
       FReg.Free;
       end;

  Except
    On E:ERegistryException do Raise;
   end;
end;

//------------------------------------------------------------------------------

Procedure TEasyRegistry.WriteBoolean(SubKey: String; Value: Boolean);
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
    FReg := TRegistry.Create;
     Try
       FReg.Rootkey := FRegistryBaseKey;
       FReg.CreateKey(FBasekey);
       FReg.OpenKey(FBaseKey, False);
       FReg.WriteBool(Subkey, Value);
     Finally
       FReg.Free;
     end;

  Except
    On E:ERegistryException do Raise;
   end;
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.ReadBoolean(Subkey: String): Boolean;
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
    FReg := TRegistry.Create;
    Try
      FReg.Rootkey := FRegistryBaseKey;
      FReg.CreateKey(FBaseKey);
      FReg.OpenKey(FBaseKey, False);
      Result := FReg.ReadBool(Subkey);
    Finally
      FReg.Free;
    end;

  Except
    On E:ERegistryException do Raise;
   end;
end;

end.
